@@ -1,4 +1,4 @@
-/* $Id: base3.c,v 1.194 2006/04/11 17:28:55 kb Exp $
+/* $Id: base3.c,v 1.194.2.1 2006/12/11 14:29:52 kb Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -631,7 +631,7 @@ basistoalg(GEN nf, GEN x)
       return gcopy(x);
     default: z=cgetg(3,t_POLMOD);
       gel(z,1) = gcopy(gel(nf,1));
-      gel(z,2) = gmul(x,pol_1[varn(nf[1])]); return z;
+      gel(z,2) = gtopoly(x, varn(nf[1])); return z;
   }
 }
 
