@@ -1,4 +1,4 @@
-/* $Id: polarit1.c,v 1.268.2.1 2006/08/03 17:21:21 kb Exp $
+/* $Id: polarit1.c,v 1.268.2.3 2006/12/13 22:33:01 kb Exp $
 
 Copyright (C) 2000-2004  The PARI group.
 
@@ -146,15 +146,23 @@ static GEN
 root_mod_4(GEN f)
 {
   long i, no, ne;
-  int z0 = !signe(constant_term(f));
-  int z2 = ((i_mod4(constant_term(f)) + 2*i_mod4(f[3])) & 3) == 0;
-  int z1, z3;
-  GEN y;
+  GEN y, t;
+  int z0, z1, z2, z3;
+
+  t = constant_term(f);
+  z0 = !signe(t);
+  z2 = ((i_mod4(t) + 2*i_mod4(f[3])) & 3) == 0;
 
   for (ne=0,i=2; i<lg(f); i+=2)
-    if (signe(f[i])) ne += mael(f,i,2);
+  {
+    t = gel(f,i);
+    if (signe(t)) ne += *int_LSW(t);
+  }
   for (no=0,i=3; i<lg(f); i+=2)
-    if (signe(f[i])) no += mael(f,i,2);
+  {
+    t = gel(f,i);
+    if (signe(t)) ne += *int_LSW(t);
+  }
   no &= 3; ne &= 3;
   z3 = (no == ne);
   z1 = (no == ((4-ne)&3));
@@ -1480,13 +1488,12 @@ Zp_appr(GEN f, GEN a)
 static GEN
 ZX_Zp_roots(GEN f, GEN p, long prec)
 {
-  GEN y, z, q, rac;
+  GEN y, z, rac;
   long lx, i, j, k;
 
   z = modulargcd(f, ZX_deriv(f));
   if (degpol(z) > 0) f = RgX_div(f,z);
-  q = equaliu(p,2)? utoipos(4): p;
-  rac = FpX_roots(f, q);
+  rac = FpX_roots(f, p);
   lx = lg(rac); if (lx == 1) return rac;
   y = cgetg(degpol(f)+1,t_COL);
   for (j=i=1; i<lx; i++)
@@ -2487,23 +2494,26 @@ FqX_factor_i(GEN f, GEN T, GEN p)
     /* u is square-free (product of irreducibles of multiplicity e) */
     nb0 = nbfact; N = degpol(u);
     t[nbfact] = FqX_normalize(u, T,p);
-    if (N == 1) nbfact++;
-    else
-    {
+    if (N) {
+      nb0 = nbfact;
+      t[nbfact] = FqX_normalize(u, T,p);
+      if (N == 1) nbfact++;
+      else
+      {
 #if 0
-      nbfact += FqX_split_Berlekamp(t+nbfact, q, T, p);
+        nbfact += FqX_split_Berlekamp(t+nbfact, q, T, p);
 #else
-      GEN P = FqX_split_Trager(t[nbfact], T, p);
-      if (P) {
-        for (j = 1; j < lg(P); j++) t[nbfact++] = gel(P,j);
-      } else {
-        if (DEBUGLEVEL) pari_warn(warner, "FqX_split_Trager failed!");
-        nbfact += FqX_sqf_split(t+nbfact, q, T, p);
-      }
+        GEN P = FqX_split_Trager(t[nbfact], T, p);
+        if (P) {
+          for (j = 1; j < lg(P); j++) t[nbfact++] = gel(P,j);
+        } else {
+          if (DEBUGLEVEL) pari_warn(warner, "FqX_split_Trager failed!");
+          nbfact += FqX_sqf_split(t+nbfact, q, T, p);
+        }
 #endif
+      }
+      for (j = nb0; j < nbfact; j++) E[j] = e;
     }
-    for (j = nb0; j < nbfact; j++) E[j] = e;
-
     if (!degpol(f2)) break;
     f = f2; df1 = df2; e += pk;
   }
