@@ -1,4 +1,4 @@
-/* $Id: polarit3.c,v 1.325.2.1 2006/08/24 21:41:42 kb Exp $
+/* $Id: polarit3.c,v 1.325.2.3 2007/03/04 22:14:36 bill Exp $
 
 Copyright (C) 2000-2005  The PARI group.
 
@@ -97,7 +97,7 @@ ZX_Z_add(GEN y, GEN x)
   gel(z,2) = addii(gel(y,2),x);
   for(i=3;i<lz;i++)
     gel(z,i) = icopy(gel(y,i));
-  if (lz==2) z = ZX_renormalize(z,lz);
+  if (lz==3) z = ZX_renormalize(z,lz);
   return z;
 }
 
@@ -3320,13 +3320,13 @@ ZX_resultant_all(GEN A, GEN B, GEN dB, ulong bound)
     bound = ZY_ZXY_ResBound(A, B, dB);
     if (bound > 50000)
     {
-      long prec = MEDDEFAULTPREC;
+      long eA = gexpo(A), eB = gexpo(B), prec = nbits2prec(max(eA,eB));
       for(;; prec = (prec-1)<<1)
       {
         GEN run = real_1(prec);
         GEN R = subres(gmul(A, run), gmul(B, run));
         bound = gexpo(R) + 1;
-        if (!gcmp0(R) || bound <= 0) break;
+        if (!gcmp0(R)) break;
       }
       if (dB) bound -= (long)(dbllog2(dB)*degA);
     }
