@@ -1,4 +1,4 @@
-/* $Id: trans1.c,v 1.211.2.1 2006/09/30 11:05:52 kb Exp $
+/* $Id: trans1.c,v 1.211.2.5 2007/03/27 15:07:03 kb Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -714,7 +714,7 @@ ser_powfrac(GEN x, GEN q, long prec)
   long e = valp(x);
   GEN y, E = gmulsg(e, q);
 
-  if (gcmp0(x)) return zeroser(varn(x), val_from_i(gceil(E)));
+  if (gcmp0(x)) return zeroser(varn(x), val_from_i(gfloor(E)));
   if (typ(E) != t_INT)
     pari_err(talker,"%Z should divide valuation (= %ld) in sqrtn",q[2], e);
   e = val_from_i(E);
@@ -1201,7 +1201,7 @@ exp1r_abs(GEN x)
     l2 += m>>TWOPOTBITS_IN_LONG;
   } else { /* rare ! */
     b = -1 - log((double)(ulong)x[2]) + (BITS_IN_LONG-1-ex)*LOG2; /*-1-log(x)*/
-    n = (long)(1 + beta/b);
+    n = (long)(1.1 + beta/b);
     m = 0;
   }
   unr=real_1(l2);
@@ -1211,15 +1211,14 @@ exp1r_abs(GEN x)
 
   s = 0; l1 = 3; av2 = avma;
   for (i=n; i>=2; i--)
-  {
+  { /* compute X^(n-1)/n! + ... + X/2 + 1 */
     setlg(X,l1); p3 = divrs(X,i);
     s -= expo(p3); p1 = mulrr(p3,p2); setlg(p1,l1);
     l1 += s>>TWOPOTBITS_IN_LONG; if (l1>l2) l1=l2;
     s &= (BITS_IN_LONG-1);
     setlg(unr,l1); p1 = addrr_sign(unr,1, p1,1);
-    setlg(p2,l1); affrr(p1,p2); avma = av2;
+    setlg(p2,l1); affrr(p1,p2); avma = av2; /* p2 <- 1 + (X/i)*p2 */
   }
-  setlg(p2,l2);
   setlg(X,l2); p2 = mulrr(X,p2);
 
   for (i=1; i<=m; i++)
@@ -1588,8 +1587,8 @@ constlog2(long prec)
   tmplog2 = newbloc(prec);
   *tmplog2 = evaltyp(t_REAL) | evallg(prec);
   av = avma;
-  n = bit_accuracy(prec) >> 1;
   l = prec+1;
+  n = bit_accuracy(l) >> 1;
   y = divrr(Pi2n(-1, l), agm1r_abs( real2n(2 - n, l) ));
   affrr(divrs(y,n), tmplog2);
   if (glog2) gunclone(glog2);
@@ -1988,6 +1987,15 @@ mpcos(GEN x)
   return gerepileuptoleaf(av, y);
 }
 
+/* convert INT or FRAC to REAL, which is later reduced mod 2Pi : avoid
+ * cancellation */
+static GEN
+tofp_safe(GEN x, long prec)
+{
+  return (typ(x) == t_INT || gexpo(x) > 0)? gadd(x, real_0(prec)) 
+                                          : fractor(x, prec);
+}
+
 GEN
 gcos(GEN x, long prec)
 {
@@ -2012,9 +2020,7 @@ gcos(GEN x, long prec)
 
     case t_INT: case t_FRAC:
       y = cgetr(prec); av = avma;
-      /* _not_ afrr: we want to be able to reduce mod Pi */
-      x = gadd(x, real_0(prec));
-      affr_fixlg(mpcos(x), y); avma = av; return y;
+      affr_fixlg(mpcos(tofp_safe(x,prec)), y); avma = av; return y;
 
     case t_INTMOD: pari_err(typeer,"gcos");
     
@@ -2079,9 +2085,7 @@ gsin(GEN x, long prec)
 
     case t_INT: case t_FRAC:
       y = cgetr(prec); av = avma;
-      /* _not_ afrr: we want to be able to reduce mod Pi */
-      x = gadd(x, real_0(prec));
-      affr_fixlg(mpsin(x), y); avma = av; return y;
+      affr_fixlg(mpsin(tofp_safe(x,prec)), y); avma = av; return y;
 
     case t_INTMOD: pari_err(typeer,"gsin");
 
@@ -2270,9 +2274,7 @@ gtan(GEN x, long prec)
 
     case t_INT: case t_FRAC:
       y = cgetr(prec); av = avma;
-      /* _not_ afrr: we want to be able to reduce mod Pi */
-      x = gadd(x, real_0(prec));
-      affr_fixlg(mptan(x), y); avma = av; return y;
+      affr_fixlg(mptan(tofp_safe(x,prec)), y); avma = av; return y;
 
     case t_PADIC:
       av = avma;
@@ -2317,9 +2319,7 @@ gcotan(GEN x, long prec)
 
     case t_INT: case t_FRAC:
       y = cgetr(prec); av = avma;
-      /* _not_ afrr: we want to be able to reduce mod Pi */
-      x = gadd(x, real_0(prec));
-      affr_fixlg(mpcotan(x), y); avma = av; return y;
+      affr_fixlg(mpcotan(tofp_safe(x,prec)), y); avma = av; return y;
 
     case t_PADIC: 
       av = avma;
