@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/gmp/mp.c"
-/* $Id: mp.c,v 1.87.2.1 2006/09/12 13:52:04 bill Exp $
+/* $Id: mp.c,v 1.87.2.3 2007/03/16 18:19:55 kb Exp $
 
 Copyright (C) 2002-2003  The PARI group.
 
@@ -126,7 +126,10 @@ setloop(GEN a)
 
 /* we had a = setloop(?), then some incloops. Reset a to b */
 GEN
-resetloop(GEN a, GEN b) { affii(b, a); return a; }
+resetloop(GEN a, GEN b) {
+  a[0] = evaltyp(t_INT) | evallg(lgefint(b));
+  affii(b, a); return a;
+}
 
 /* assume a > 0, initialized by setloop. Do a++ */
 static GEN
@@ -1137,14 +1140,14 @@ resmod2n(GEN x, long n)
   { /* strip leading zeroes from result */
     xd--; while (k && !*xd) { k--; xd--; }
     if (!k) return gen_0;
-    ly = k+2; xd++;
+    ly = k+2;
   }
   else
     ly = k+3;
 
   zd = z = cgeti(ly);
   *++zd = evalsigne(1) | evallgefint(ly);
-  for ( ;k; k--) *++zd = *++xd;
+  xd = x+1; for ( ;k; k--) *++zd = *++xd;
   if (hi) *++zd = hi;
   return z;
 }
