@@ -1,5 +1,5 @@
 #line 2 "../src/kernel/none/mp_indep.c"
-/* $Id: mp_indep.c,v 1.33 2006/01/31 13:22:44 kb Exp $
+/* $Id: mp_indep.c,v 1.33.2.1 2006/05/31 08:30:49 kb Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -566,7 +566,7 @@ remiimul(GEN x, GEN sy)
   q = truncr(q); /* differs from divii(x, y) at most by 1 */
   r = subii(x, mulii(y,q));
   if (signe(r) < 0)
-    r = addiispec(r+2,y+2, lgefint(r)-2, lgefint(y)-2);
+    r = subiispec(y+2,r+2, lgefint(y)-2, lgefint(r)-2); /* y - |r| */
   else
   {
     /* remii(x,y) + y >= r >= remii(x,y) */
