@@ -1,4 +1,4 @@
-/* $Id: members.c,v 1.24 2006/03/15 09:26:40 kb Exp $
+/* $Id: members.c,v 1.24.2.1 2006/11/16 16:18:31 kb Exp $
 
 Copyright (C) 2000-2003  The PARI group.
 
@@ -206,9 +206,10 @@ member_diff(GEN x) /* different */
 GEN
 member_codiff(GEN x) /* codifferent */
 {
-  long t; GEN nf = get_nf(x,&t), y = nfmats(nf);
+  long t; GEN H, nf = get_nf(x,&t), y = nfmats(nf);
   if (!y) member_err("codiff");
-  return gdiv(gel(y,6), absi(gel(nf,3)));
+  H = hnf(gel(y,6));
+  return gdiv(H, gcoeff(H,1,1));
 }
 
 GEN
