@@ -1,4 +1,4 @@
-/* $Id: thue.c,v 1.76 2006/04/11 17:28:55 kb Exp $
+/* $Id: thue.c,v 1.76.2.1 2007/03/10 00:04:13 kb Exp $
 
 Copyright (C) 2000  The PARI group.
 
@@ -476,15 +476,19 @@ GuessQi(GEN b, GEN c, GEN *eps)
 static GEN
 MiddleSols(GEN *pS, GEN bound, GEN roo, GEN poly, GEN rhs, long s, GEN c1) 
 {
-  long j, k, d = degpol(poly); 
+  long j, k, nmax, d = degpol(poly); 
   GEN bndcf = sqrtnr(shiftr(c1,1), d - 2); 
 
   if (cmprr(bound, bndcf) == -1) return bound; 
+  /* divide by log((1+sqrt(5))/2) 
+   * 1 + ==> ceil
+   * 2 + ==> continued fraction is normalized if last entry is 1 */
+  nmax = 2 + (long)(gtodouble(mplog(bound)) / 0.4812118250596);
+  if (nmax < 3) nmax = 3;
 
   for (k = 1; k <= s; k++) 
   {
-    GEN rr = greal(gel(roo,k)); 
-    GEN t = contfrac0(rr, 0, 100); 
+    GEN t = contfrac0(real_i(gel(roo,k)), NULL, nmax); 
     GEN p, q, pm1, qm1, p0, q0, z; 
 
     pm1 = gen_0; p0 = gen_1; 
